﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.DeploymentService.AdminClient {
  public partial class PluginListView : UserControl {
    public event ItemCheckedEventHandler ItemChecked;

    private IEnumerable<IPluginDescription> plugins;
    public IEnumerable<IPluginDescription> Plugins {
      get { return plugins; }
      set {
        plugins = value;
        UpdateControls();
      }
    }

    public IEnumerable<IPluginDescription> CheckedPlugins {
      get {
        return from item in listView.Items.OfType<ListViewItem>()
               where item.Checked
               select item.Tag as IPluginDescription;
      }
    }

    private bool suppressItemCheckedEvents = false;

    public PluginListView() {
      InitializeComponent();
    }

    public void CheckPlugin(IPluginDescription plugin) {
      var items = FindItemsForPlugin(plugin);
      if (items.Count() == 0) throw new ArgumentException();
      var item = items.Single();
      if (!item.Checked) {
        suppressItemCheckedEvents = true;
        item.Checked = true;
        suppressItemCheckedEvents = false;
      }
    }

    private void UpdateControls() {
      if (plugins != null) {
        listView.Items.Clear();
        suppressItemCheckedEvents = true;
        foreach (var plugin in plugins) {
          listView.Items.Add(CreateListViewItem(plugin));
        }
        suppressItemCheckedEvents = false;
      }
    }

    private ListViewItem CreateListViewItem(IPluginDescription plugin) {
      var item = new ListViewItem(new string[] { plugin.Name, plugin.Version.ToString() });
      item.Tag = plugin;
      return item;
    }

    private void listView_ItemChecked(object sender, ItemCheckedEventArgs e) {
      if (suppressItemCheckedEvents) return;
      if (e.Item.Checked) {
        // also check all dependencies
        suppressItemCheckedEvents = true;
        var dependencies = GetAllDependencies(e.Item.Tag as IPluginDescription);
        foreach (var dep in dependencies) {
          var item = FindItemsForPlugin(dep).Single();
          if (!item.Checked) item.Checked = true;
        }
        suppressItemCheckedEvents = false;
      } else {
        // also uncheck all dependencies
        suppressItemCheckedEvents = true;
        var dependents = GetAllDependents(e.Item.Tag as IPluginDescription);
        foreach (var dep in dependents) {
          var item = FindItemsForPlugin(dep).Single();
          if (item.Checked) item.Checked = false;
        }
        suppressItemCheckedEvents = false;
      }
      OnItemChecked(e);
    }

    private IEnumerable<ListViewItem> FindItemsForPlugin(IPluginDescription plugin) {
      return from item in listView.Items.OfType<ListViewItem>()
             let p = item.Tag as IPluginDescription
             where p.Name == plugin.Name
             where p.Version == plugin.Version
             select item;
    }

    private IEnumerable<IPluginDescription> GetAllDependents(IPluginDescription plugin) {
      return from p in plugins
             let matchingEntries = from dep in GetAllDependencies(p)
                                   where dep.Name == plugin.Name
                                   where dep.Version == plugin.Version
                                   select dep
             where matchingEntries.Any()
             select p;
    }

    private IEnumerable<IPluginDescription> GetAllDependencies(IPluginDescription plugin) {
      foreach (var dep in plugin.Dependencies) {
        foreach (var recDep in GetAllDependencies(dep)) {
          yield return recDep;
        }
        yield return dep;
      }
    }

    private void OnItemChecked(ItemCheckedEventArgs e) {
      if (ItemChecked != null && !suppressItemCheckedEvents) ItemChecked(this, e);
    }
  }
}
